// examples:
// java BigFermatTest 479001599
// java BigFermatTest 87178291199
// java BigFermatTest 170141183460469231731687303715884105727

import java.math.BigInteger;
import java.util.Random;

public class BigFermatTest {

    public static final BigInteger ZERO = BigInteger.ZERO;
    public static final BigInteger ONE = BigInteger.ONE;
    public static final BigInteger TWO = new BigInteger("2");
    public static final Random generator = new Random();

    public static void main(String[] args) {
        if (args.length != 1) {
            System.out.println("Usage: java BigFermatTest <N>");
            return;
        }
	BigInteger n = new BigInteger(args[0]);
	if (fermatTest(n) == true) {
	    System.out.printf("%d is very likely prime\n", n);
	} else {
	    System.out.printf("%d is definitely not prime\n", n);
	}
    }

    public static boolean fermatTest(BigInteger n) {
	if (n.compareTo(TWO) < 0) return false;  // n < 2
	int TRIALS = 10;
	for (int i = 0; i < TRIALS; i++) {
	    // choose a random number a in the range 1 <= a <= n-1
	    BigInteger a = pickRandom(ONE, n.subtract(ONE));
	    if (fastpowerModulo(a, n, n).compareTo(a) != 0) return false;
	}
	return true;
    }

    // fast exponentiation, modulo M
    public static BigInteger fastpowerModulo(BigInteger b, BigInteger n, BigInteger M) {
	if (n.compareTo(ZERO) == 0) {
	    return ONE;
	} else if (isEven(n)) {
	    return square(fastpowerModulo(b, n.divide(TWO), M)).remainder(M);
	} else {
	    return times(b, fastpowerModulo(b, n.subtract(ONE), M)).remainder(M);
	}
    }

    public static boolean isEven(BigInteger n) {
	return (n.remainder(TWO).compareTo(ZERO) == 0);
    }

    public static BigInteger square(BigInteger n) {
	return n.multiply(n);
    }

    public static BigInteger times(BigInteger b, BigInteger n) {
	return b.multiply(n);
    }

    // return a random BigInteger in the range start to end, inclusive
    public static BigInteger pickRandom(BigInteger start, BigInteger end) {
        BigInteger range = end.subtract(start);
        BigInteger r = new BigInteger(range.bitLength(), generator);
        while (r.compareTo(range) > 0) {
            r = new BigInteger(range.bitLength(), generator);  // try again
        }
        return start.add(r);
    }
}
