/*
Probabilistic prime test based on Fermat's Little Theorem:

If N is a prime number, then the relation

     N
   a   modulo N = a

holds for all numbers from 1 to N-1, inclusive.  On the other hand, if N is not
prime, then USUALLY MOST numbers from 1 to N-1 will not satisfy this relation.

Idea: pick a number from 1 to N-1 at random and see if the relation holds.  If
it fails, we know N isn't prime.  If it passes, N is PROBABLY prime, but try a
few more spot checks to be more certain.
*/

public class FermatTest {

    public static void main(String[] args) {
        if (args.length != 1) {
            System.out.println("Usage: java FermatTest <N>");
            return;
        }
	long n = Long.parseLong(args[0]);
	if (primeTest6(n) == true) {
	    System.out.printf("%d is very likely prime\n", n);
	} else {
	    System.out.printf("%d is definitely not prime\n", n);
	}
    }

    // the Fermat test
    public static boolean primeTest6(long n) {
        return false; // to be completed
    }
    
    // fast exponentiation
    public static long fastpower(long b, long n) {
	if (n == 0) {
	    return 1;
	} else if (isEven(n)) {
	    return square(fastpower(b, n/2));
	} else {
	    return times(b, fastpower(b, n-1));
	}
    }

    public static boolean isEven(long n) {
	return (n % 2 == 0);
    }

    public static long square(long n) {
	return n * n;
    }

    public static long times(long b, long n) {
	return b * n;
    }

    // return a random integer in the range start to end, inclusive
    public static long pickRandom(long start, long end) {
	return start + Math.round(Math.random() * (end - start));
    }
}
