// This program counts primes quietly, using the specified prime test
// up to the specified limit
//
// Usage: java Quietcount <1-6> <limit>

import java.util.ArrayList;
import java.io.*;

public class Quietcount {

    // used by primeTest5
    private static ArrayList<Long> primeList = new ArrayList<Long>();
    
    public static void main(String[] args) throws IOException {
        if (args.length != 2) {
            System.out.println("Usage: java Quietcount <1-6> <limit>");
            return;
        }
        int whichTest = Integer.parseInt(args[0]);
        long limit = Long.parseLong(args[1]);
        int numPrimes = 0;
        long biggest = 0;
        primeList.clear();
        System.out.printf("Counting primes using primeTest%d...\n", whichTest);
        for (long n = 2; n <= limit; n++) {
            if (primeTest(whichTest, n) == true) {
                numPrimes++;
                biggest = n;
                primeList.add(n);
            }
        }
        System.out.printf("Found %d primes less than or equal to %d\n", numPrimes, limit);
        System.out.printf("Biggest prime is %d\n", biggest);
    }

    public static boolean primeTest(int whichTest, long n) {
        if (whichTest == 1) return primeTest1(n);
        if (whichTest == 2) return primeTest2(n);
        if (whichTest == 3) return primeTest3(n);
        if (whichTest == 4) return primeTest4(n);
        if (whichTest == 5) return primeTest5(n);
        if (whichTest == 6) return primeTest6(n);
        return false;
    }
    //-------------------------------------------------------------------

    public static boolean primeTest1(long n) {
        if (n < 2) return false;
        for (long i = 2; i < n; i++) {
            if (n % i == 0) return false;
        }
        return true;
    }
    //-------------------------------------------------------------------

    public static boolean primeTest2(long n) {
        if (n < 2) return false;
        for (long i = 2; i <= n/2; i++) {
            if (n % i == 0) return false;
        }
        return true;
    }
    //-------------------------------------------------------------------

    public static boolean primeTest3(long n) {
        if (n < 2) return false;
        long squareRoot = (long) Math.sqrt(n);
        for (long i = 2; i <= squareRoot; i++) {
            if (n % i == 0) return false;
        }
        return true;
    }
    //-------------------------------------------------------------------

    public static boolean primeTest4(long n) {
        if (n < 2) return false;
        if (n == 2) return true;
        if (n % 2 == 0) return false;
        long squareRoot = (long) Math.sqrt(n);
        for (long i = 3; i <= squareRoot; i += 2) {
            if (n % i == 0) return false;
        }
        return true;
    }
    //-------------------------------------------------------------------

    public static boolean primeTest5(long n) {
        if (n < 2) return false;
        if (n == 2) return true;
        if (n % 2 == 0) return false;
        long squareRoot = (long) Math.sqrt(n);
        for (Long i : primeList) {
            if (i > squareRoot) return true;  // only check up to sqrt(n)
            if (n % i == 0) return false;
        }
        return true;
    }
    //-------------------------------------------------------------------
    // the Fermat test

    public static boolean primeTest6(long n) {
        int TRIALS = 10;
        if (n < 2) return false;
        for (int i = 1; i <= TRIALS; i++) {
            long a = pickRandom(1, n-1);
            if (fastpowerModulo(a, n, n) != a) return false;
        }
        return true;
    }

    // fast exponentiation, modulo M
    public static long fastpowerModulo(long b, long n, long M) {
        if (n == 0) {
            return 1;
        } else if (isEven(n)) {
            return square(fastpowerModulo(b, n/2, M)) % M;
        } else {
            return times(b, fastpowerModulo(b, n-1, M)) % M;
        }
    }

    public static boolean isEven(long n) {
        return (n % 2 == 0);
    }

    public static long square(long n) {
        return n * n;
    }

    public static long times(long b, long n) {
        return b * n;
    }

    // return a random integer in the range start to end, inclusive
    public static long pickRandom(long start, long end) {
        return start + (long) Math.round(Math.random() * (end - start));
    }
    //-------------------------------------------------------------------
}
